#!/bin/perl5.8

=head1 Name

SSM::SSM1985 --- Subroutines for SSM 1985 data

	Created: 2011-01-30
	Updated: 2011-01-30

=head1 Usage

use SSM::SSM1985

=cut

package SSM::SSM1985;

use Dredu;

use Exporter ();
@ISA = qw(Dredu Exporter);
@EXPORT = qw( Eduyear Income_10000yen Status4 %eduyear Id );
@EXPORT_OK = qw( );


# ************************************
#  Declare Constants and Global variables
# ************************************

=head1 Global Variables

=head2 %kind

=cut

%kind = (
	1 => 'A',
	2 => 'B',
	3 => 'F',
);

=head2 %eduyear

=cut

%eduyear = (
	 1,	 6,		# (old) standard elementary
	 2,	 8,		# (old) high elementary
	 3,	11,		# (old) middle school
	 4,	14,		# (old) high school
	 5,	17,		# (old) university
	12,	 9,		# (new) junior-high
	13,	12,		# (new) high schoool
	14,	14,		# (new) junior college
	15,	16,		# (new) university
	20,	'',		# other
	30,	 0,		# no education
) ;

=head2 %status4

=cut

%status4 = (
	1 => 1, 	# Managers
	2 => 2, 	# Regular Employee
	3 => 3, 	# Non-Regular Employee
	4 => 1, 	# Self-employed without employee
	5 => 1, 	# Self-employed with employee
	6 => 1, 	# Family worker of self-employed
	7 => 3, 	# Home Pieceworkers
	8 => 9, 	# Student
	9 => 9, 	# No job
	10=> 9, 	# Military service
) ;

# ************************************
# Methods
# ************************************

=head1 Exported Subroutines


=head2 Kind() - distinguish between the questionnaires

	return: 1(Male A) / 2(Male B) / 3(Female)

=cut


sub Kind {
	my($self) = shift;
	return $self->Value( 'KIND1' ) if $self;
	return $main::KIND1 ;
}

=head2 Kind_p() - Prefix for each questionnaire

	return: A(Male A) / B(Male B) / F(Female)

=cut


sub Kind_p {
	my($self) = shift;
	my $k ;
	if( $self ){
		$k = $self->Value( 'KIND1' ) 
	}
	else{
		$k = $main::KIND1 ;
	}
	return $kind{$k}  if exists $kind{$k};
	warn( "Invalid questionnaire: $k." );
}

=head2 Id() - Identification Number

	return: 6-digit ( kind + id )

=cut

sub Id {
	my($self) = shift;
	if( $self ){
		return sprintf "%d%03d%02d" ,  $self->values( qw/KIND1 IDT1 IDP1/) if 3==$self->Kind;
		return sprintf "%d%05d" ,  $self->values( 'KIND1' , 'ID1') ;
	
	}
	return sprintf "%d%03d%02d" ,  $main::KIND1, $main::IDT1, $main::IDP1 if 3==$self->Kind;
	return sprintf "%d%05d" ,  $main::KIND1 , $main::ID1;
}

=head2 Sex() - return Sex

	return: 1(male) / 2(female)

=cut

sub Sex {
	my($self) = shift;
	return (3==$self->Kind)   	? 2 : 1  if self;
	return (3==$main::KIND1)  	? 2 : 1 ;
}


=head2 Qvalue( $v ) - convert variable names according to the questionnaire

	$v: Name of the valiable, with the first character (A, B, or F) ommitted
	return: Value of the valiable

=cut

sub Qvalue {
	my($self, $a, $b ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	my $p = $self->Kind_p;
	$a = "$p$a";
	return $self->Value($a) if '' eq $b;
	$self->Value($a, $b);
}

=head2 qvalues( @v ) - convert variable names according to the questionnaire

	@v: Array of the names of the valiables, with the first character (A, B, or F) ommitted
	return: Array of values of the valiables

=cut

sub qvalues {
	my($self, @a) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	my $p = $self->Kind_p;
	@a = map{ "$p$a" } @a;
	$self->values(@a);
}


=head2 Age_marriage() - how old was the respondent at the marriage

=cut

sub Age_marriage {
	my ( $self, @a ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	return $self->Value('MarAge');
}

=head2 income() - income

	return: array for income( Household, Personal, Spuse)

=cut

sub income {
	my ( $self, @a ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	
	return $self->values( 'A25A', 'A25B', 'A25C' ) if 1==$self->Kind;;
	return $self->values( 'B25A', 'B25B', 'B25C' ) if 2==$self->Kind;;
	return $self->values( 'F27A', 'F27B', 'F27C' ) if 3==$self->Kind;
	warn("Invalid KIND of questionnaire.")
}

=head2 Income_10000yen($x) - Metric of income (x 10,000 yen)

	$x: SSM 1985 income code

=cut

%income_10000yen =(
	  1=> 0,
	  2=> (25)/2,
	  3=> (25+50)/2,
	  4=> (50+75)/2,
	  5=> (75+100)/2,
	  6=> (100+125)/2,
	  7=> (125+150)/2,
	  8=> (150+175)/2,
	  9=> (175+200)/2,
	 10=> (200+225)/2,
	 11=> 250 ,
	 12=> 300 ,
	 13=> 350 ,
	 14=> 400,
	 15=> 450,
	 16=> 500,
	 17=> 550,
	 18=> 600,
	 19=> 650,
	 20=> 700,
	 21=> 750,
	 22=> 800,
	 23=> 850,
	 24=> 900,
	 25=> 950,
	 26=>1000,
	 27=>1050,
	 28=>1100,
	 29=>1150,
	 30=>1200,
	 31=>1250 ,
	 32=>1300 ,
	 33=>1350 ,
	 34=>1400 ,
	 35=>1450 ,
	 36=>1500 ,
);


# To convert income
sub Income_10000yen {
	my $x = @_[0];
	return $x if miss($x);

	if( exists $income_10000yen{$x} ){
		return $income_10000yen{$x};
	}
	elsif(36<$x) {
		return 1500 + ( $x-36 )*50
	}
	else {
		warn( "Illegal value for income: $x\n" );
		Missval( 'BLANK' );
	}
}

	# -------- int &years_old( int x );
	# ԒlFx ` x+1 NO̖{lN
	#	̒qNȂǂƖ{lNƂ̍ƂB
	#	a񂪂Ȃ̂ŁAȏ㒲Ȃ
	# &age_fc, &age_lc Ăяo
sub years_old {
	my ( $self, @a ) = @_;
	die ( "This is an instance method, not for $self.")  unless ref($self);
	foreach(@a){
		next if miss($_);
		$_ = ${ $self->{var}{Age} } - $_ ;	# {lNƂ̍Ƃ
	}
	return $a[0] if 1==@a;
	@a;
}

# ************************************
# Subroutines
# ************************************

=head2 Eduyear($e) - Return duration of standard school education

	$e: SSM education code

=cut

sub Eduyear {
	return $_[0]  if miss($_[0]);
	return $eduyear{$_[0]} if exists $eduyear{$_[0]} ;
	warn ( "Invalid education code: $_[0]\n" );
	undef;
}

=head2 Status4($e) - Return classification(4) for employment status

	$e: SSM 1985 employment status

=cut

sub Status4 {
	return $_[0]  if miss($_[0]);
	return $status4{$_[0]} if exists $status4{$_[0]} ;
	warn ( "Invalid Employment Status: $_[0]\n" );
	undef;
}

1;


=head1 Author

=over 4

=item B<TANAKA Sigeto> <tsigeto@nik.sal.tohoku.ac.jp>

Graduate School of Arts and Letters, Tohoku University

Kawauti 27-1, Aoba Ku, Sendai, Miyagi, 980-8576 JAPAN

WWW: http://www.sal.tohoku.ac.jp/~tsigeto/

=back

=head1 Copyright

	Copyright (c) 2011 TANAKA Sigeto

You may use, duplicate, revise, or redistribute 
this script, without permission from the copyright holder.

=cut
